# Solana: Generate Address

## Requirements

{{ #include ../../../../operator-requirements.md:requirements }}

{{ #include ../../../../../../component-documents/linux-workstation.md:content }}

* [High Visibility Storage](TODO): plastic container or bag that's used to keep items while not in use in a visible location like the middle of a desk.

* [Quorum PGP key pairs](../../../key-types.md#quorum-pgp-keypair)

* [Ceremony SD card](../../../ceremony-sd-card-provisioning.md)

## Procedure

1. Enter the designated location with the quorum of operators and all required equipment

1. Lock access to the location - there should be no inflow or outflow of people during the ceremony

1. Place Ceremony SD card in High Visibility Storage

1. Retrieve sealed Air-Gapped bundle, polaroid of tamper evidence, and online laptop from locked storage

{{ #include ../../../../../../component-documents/tamper-evidence-methods.md:vsbwf-procedure-unsealing}}

1. Place all contents except for the laptop into High Visibility Storage

### Offline Machine: Generate Address

{{ #include ../template-gen-address-0.md:content }}

1. Generate a new address:
	```
    $ icepick workflow sol generate-address --account $account_id > $account_id.json
	```

{{ #include ../template-gen-address-1.md:content }}

### Online Machine: Generate Nonce Account

1. Turn on online machine

1. Retrieve the Ceremony SD card from High Visibility Storage and plug it into the computer

1. {{ #include ../../../../../../component-documents/finding-device-name.md:content }}

1. Copy the `vaults` repository from the Ceremony SD card:
	```
    $ cp -r /media/vaults ~/
	```
    * If the `~/vaults/` repository already exists, ensure it doesn't have any changes that haven't been committed, then remove it using `sudo rm -rf ~/vaults` before re-running the previous step

1. Ensure `keyfork` is available on the system:

    * Follow steps from [installation guide](https://git.distrust.co/public/keyfork#install)

1. Ensure `icepick` is available on system

    * Follow steps from [installation guide](TODO)

1. Set unsafe `keyfork` usage variable:
	```
    $ export INSECURE_HARDWARE_ALLOWED=1
	```
	* [Bug: user shouldn't have to set this insecure config. Issue #34 fixes this.](https://git.distrust.co/public/icepick/issues/34)
1. Generate throwaway mnemonic to generate address which will be used for funding the creation of nonce account:
	```
    $ keyfork mnemonic generate | KEYFORK_PROMPT_TYPE=headless keyfork recover mnemonic --daemon
	```
1. Change directory into the desired \<namespace>/\<coin> directory:
	```
    $ cd ~/vaults/<namespace>/<coin>
	```
1. Select which account you are creating the delegate address by viewing the appropriate \<namespace>/\<coin>/ directory:
	```
    $ ls -la .
	```
1. Once you have selected the appropriate account, set the account_id variable:
	```
    $ account_id=<num>
	```
1. Use `icepick` to generate nonce account:

    * The following command will need to be updated to use the appropriate \<cluster>, which can be `devnet`, `testnet` or `mainnet-beta`

    * Set `icepick` config file:
	```
    $ export ICEPICK_CONFIG_FILE=<path_to_icepick_repositry>/icepick.toml`
	```
	```
    $ icepick workflow sol generate-nonce-account --authorization-address "$(cat $account_id.txt)" --input-file $account_id.json > $account_id-na.json
	```
    * Repeat command if returned message is "The transaction was possibly not received by the cluster."

1. Airdrop the wallet displayed on-screen with 0.01 SOL

    * Once the airdrop is done, nonce account will be created

1. Stage, commit, sign and push the changes:
    ```
    $ git add .
    $ git commit -m -S "<message>"
    $ git push origin HEAD
    ```
### Sealing

1. Gather all the original items that were in the air-gapped bundle:

    * Air-gapped computer

    * AirgapOS SD card

{{ #include ../../../../../../component-documents/tamper-evidence-methods.md:vsbwf-procedure-sealing}}

